# Exercise Gelido (from Ghiani, Laporte, Musmanno textbook, p.93)
# DATA
set Sites;
set Markets;

param dist {Sites,Markets};	    # Distances between sites and markets [miles]
param f1;		                # Fixed cost first range [$/year]
param f2;	        	        # Fixed cost second range [$/year]
param g1;		                # Variable cost first range [$/hundred kg]
param g2;	                	# Variable cost second range [$/hundred kg]
param capacity1;             	# Capacity of the sites, first range [hundred kg/year]
param capacity2;             	# Capacity of the sites, second range [hundred kg/year]
param demand {Markets};			# Markets demand [hundred kg/year]
param cost_per_mile;			# Cost per kilometer [$/mile]
param truck_cap;				# Truck capacity [hundred kg]

# VARIABLES
var y1 {Sites} binary;			# Site selection variables: y=1 used; y=0 not used [first range]
var y2 {Sites} binary;			# Site selection variables: y=1 used; y=0 not used [second range]
var x1 {Sites,Markets} >=0;		# Flow variables: amount of product transported, first range [hundred kg/year]
var x2 {Sites,Markets} >=0;		# Flow variables: amount of product transported, second range [hundred kg/year]

# CONSTRAINTS
# Constraints on production capacity for each plant and range [hundred kg/year] 
subject to Prod_cap1 {i in Sites}:
  sum {j in Markets} x1[i,j] <= capacity1 * y1[i];
subject to Prod_cap2 {i in Sites}:
  sum {j in Markets} x2[i,j] <= capacity2 * y2[i];

# Constraints on demand satisfaction for each market [hectoliters/year]
subject to Demand_satisfaction {j in Markets}:
  sum {i in Sites} (x1[i,j] + x2[i,j]) = demand[j];

# OBJECTIVE
#Minimization of total costs: fixed and variable costs at the plants + transportation costs [$/year]
minimize z: sum {i in Sites} (f1 * y1[i] + f2 * y2[i])
          + sum {i in Sites, j in Markets} (g1 * x1[i,j] + g2 * x2[i,j])
          + sum {i in Sites, j in Markets} (cost_per_mile*2*dist[i,j]/truck_cap) * (x1[i,j] + x2[i,j]);

###############################
data;

set Sites := Linares Monclova Monterrey;
set Markets := Bustamante Saltillo St_Catarina Montemorelos;

param dist:	Bustamante	Saltillo	St_Catarina	Montemorelos :=
Linares	    165.0		132.5		 92.7		 32.4
Monclova	 90.8		118.5		139.0		176.7
Monterrey	 84.2		 51.6		 11.9		 49.5;
 
param demand :=
Bustamante	    6200
Saltillo	    6600
St_Catarina 	5800
Montemorelos    4400;

param f1 :=  82252;
param f2 := 134400;
param g1 := 18.5;
param g2 :=  4.1;
param capacity1 :=  3500;
param capacity2 := 20000;
param cost_per_mile := 0.98;
param truck_cap := 10;

end;
