# Exercise Goutte (from Ghiani, Laporte, Musmanno textbook, p.87)
# DATA
set Markets;
set Plants;
param dist {Plants,Markets};	# Distances between plants and markets [km]
param fixed_cost {Plants};		# Fixed costs for operating plants [$/year]
param capacity {Plants};		# Production capacity [hectoliters/year]
param demand {Markets};			# Markets demand [hectoliters/year]
param max_dist;					# Maximum allowed distance between plant and market [km]
param cost_per_km;				# Cost per kilometer [$/km]
param truck_cap;				# Truck capacity [hectoliters]

# VARIABLES
var y {Plants} binary;			# Selection variables: y=1 used; y=0 not used
var x {Plants,Markets} >=0;		# Flow variables: amount of product transported [hectoliters/year]

# CONSTRAINTS
# Constraints on production capacity for each plant [hectoliters/year] 
subject to Prod_cap {i in Plants}:
  sum {j in Markets} x[i,j] <= capacity[i] * y[i];

# Constraints on demand satisfaction for each market [hectoliters/year]
subject to Demand_satisfaction {j in Markets}:
  sum {i in Plants} x[i,j] = demand[j];

# Constraints on the maximum distance
subject to Too_far {i in Plants, j in Markets: dist[i,j] > max_dist}:
  x[i,j] = 0;

# OBJECTIVE
#Minimization of total costs: fixed costs at the plants + transportation costs [$/year]
minimize z: sum {i in Plants} fixed_cost[i] * y[i]
          + sum {i in Plants, j in Markets} (cost_per_km * 2 * dist[i,j]/truck_cap) * x[i,j];

###############################
data;

set Markets := Brossard Granby St_Julie Sherbrooke Valleyfield Verdun;
set Plants := Brossard Granby LaSalle Mascouche Montreal St_Julie Sherbrooke Terrebonne Valleyfield Verdun;

param dist:	Brossard	 Granby		St_Julie	Sherbrooke	Valleyfield	Verdun :=
Brossard	  0.0		 76.1		 30.4		139.4		 72.6		 11.7
Granby		 76.1		  0.0		 71.0		 77.2		144.5		 83.7
LaSalle		 20.8		 92.9		 47.2		156.1		 47.5		 11.7
Mascouche	 54.7		113.3		 52.9		187.2		 93.0		 45.2
Montreal	 13.5		 85.5		 28.0		148.7		 67.3		  9.3
St_Julie	 30.4		 71.0		  0.0		138.2		 94.5		 38.1
Sherbrooke	139.4		 77.2		138.2		  0.0		207.9		146.9
Terrebonne	 47.8		106.5		 46.2		180.2		 86.7		 38.9
Valleyfield	 72.6		144.5		 94.5		207.9		  0.0		 63.4
Verdun		 11.7		 83.7		 38.1		146.9		 63.4		  0.0;
 
param:		fixed_cost	capacity :=
Brossard	81400		22000
Granby		83800		24000
LaSalle		88600		28000
Mascouche	91000		30000
Montreal	79000		20000
St_Julie	86200		26000
Sherbrooke	88600		28000
Terrebonne	91000		30000
Valleyfield	79000		20000
Verdun		80200		21000;

param demand :=
Brossard	14000
Granby		10000
St_Julie	 8000
Sherbrooke	12000
Valleyfield	10000
Verdun		 9000;

param max_dist := 99999;
param cost_per_km := 0.92;
param truck_cap := 150;

end;
