# Exercise K9 (from Ghiani, Laporte, Musmanno textbook, p.104)
# DATA
set Plants := 1..2;
set Sites := 1..4;
set Products := 1..2;
set Markets := 1..3;

param dist {Plants,Sites,Markets};	# Distances [km]
param p;                            # Number of sites to be selected
param minlevel;                     # Minimum flow [hl/y]
param maxlevel;                     # Maximum flow [hl/y]
param f;		                    # Fixed cost [/y]
param g;                            # Marginal cost [/hl]
param unit_cost {Products};	        # Cost per kilometer and hectoliter [/(km*hl)]
param demand {Markets,Products};	# Markets demand [hl/y]
param capacity {Plants,Products};   # Production capacity [hl/y]

# VARIABLES
var y {Sites} binary;			            # Site selection variables: y=1 used; y=0 not used
var x {Plants,Sites,Markets,Products} >=0;	# Flow variables: amount of product transported [hl/y]

# CONSTRAINTS
# Number of sites to be selected
subject to Number:
  sum {j in Sites} y[j] = p;

# Constraints on total flow through the sites [hl/y]
subject to MinFlow {j in Sites}:
  sum {i in Plants, k in Markets, w in Products} x[i,j,k,w] >= minlevel * y[j];
subject to MaxFlow {j in Sites}:
  sum {i in Plants, k in Markets, w in Products} x[i,j,k,w] <= maxlevel * y[j];

# Constraints on production capacity for each plant and product [hl/y] 
subject to Prod_cap {i in Plants, w in Products}:
  sum {j in Sites, k in Markets} x[i,j,k,w] <= capacity[i,w];

# Constraints on demand satisfaction for each market and product [hl/y]
subject to Demand_satisfaction {k in Markets, w in Products}:
  sum {i in Plants, j in Sites} x[i,j,k,w] = demand[k,w];

# OBJECTIVE
#Minimization of total costs: fixed and variable costs at the selected sites + transportation costs [/y]
minimize z: sum {j in Sites} f * y[j]
          + sum {j in Sites} g * sum {i in Plants, k in Markets, w in Products} x[i,j,k,w]
          + sum {i in Plants, j in Sites, k in Markets, w in Products} 
          (unit_cost[w]*dist[i,j,k]) * x[i,j,k,w];

###############################
data;

param dist :=
[1,1,1]  400
[1,2,1]  600
[1,3,1] 1000
[1,4,1] 1600
[2,1,1]  800
[2,2,1]  800
[2,3,1]  800
[2,4,1] 1200
[1,1,2]  600
[1,2,2]  400
[1,3,2]  600
[1,4,2] 1200
[2,1,2] 1000
[2,2,2]  600
[2,3,2]  400
[2,4,2]  800
[1,1,3] 1100
[1,2,3]  900
[1,3,3]  900
[1,4,3]  900
[2,1,3] 1500
[2,2,3] 1100
[2,3,3]  700
[2,4,3]  500;

param demand:   1       2 :=
1               800000  300000
2               600000  400000
3               700000  500000;

param capacity: 1       2 :=
1               1200000 500000
2               1500000 800000;

param p := 2;
param minlevel := 1000000;
param maxlevel := 2500000;
param f := 10000000;
param g := 0.25;

param unit_cost := 
1 0.67
2 0.82;

end;
