# Exercise p-medians

# DATA
param nV;			# Number of vertices
set V := 1..nV;		# Set of vertices
param d {V,V};		# Distances between vertices
param p;			# Number of medians to be located

# VARIABLES
var y {V} binary;		# Location variables
var x {V,V} >=0, <=1;	# Allocation variables

# CONSTRAINTS
# Constraint on the number of allowed medians
subject to Number_of_medians:
  sum {i in V} y[i] = p;

# Constraint on the number of assignments for each vertex
subject to Assignments {j in V}:
  sum {i in V} x[i,j] = 1;
  
# Relationship between x and y
subject to x_y {i in V, j in V}:
  x[i,j] <= y[i];

# OBJECTIVE
minimize z: sum {i in V, j in V} d[i,j] * x[i,j];

#########
data;

param nV := 11;

param d: 	1	2	3	4	5	6	7	8	9	10	11 :=
1			0	12	18	23	15	13	11	13	14	6	8
2			12	0	9	19	15	16	13	9	8	9	13
3			18	9	0	11	11	12	9	5	4	12	16
4			23	19	11	0	9	11	14	10	11	17	17
5			15	15	11	9	0	2	5	6	7	10	7
6			13	16	12	11	2	0	3	7	8	8	5
7			11	13	9	14	5	3	0	4	5	5	8
8			13	9	5	10	6	7	4	0	1	7	11
9			14	8	4	11	7	8	5	1	0	8	12
10			6	9	12	17	10	8	5	7	8	0	4
11			8	13	16	17	7	5	8	11	12	4	0;

param p := 3;

end;
